class Worker extends Dude
{
  int jobType;            // 0 = farmer, 1 = miner;
  boolean returnHome;     // Whether this guy is returning home from his job or vica versa
  int homeX, homeY;       // Location of this Person's 'home' (where they first spawned)
  int jobX, jobY;         // Location of this Person's 'job'

  int food;               // Amount of food held by this character
  int minerals;           // Amount of minerals held by the character

  boolean foundPath;

  Worker(int x_, int y_, int hx_, int hy_, int jx_, int jy_, int jobType_)
  {
    x = x_;
    y = y_;
    
    jobType = jobType_;
    
    if (tiles[x+y*terrainMap.width].terrainType == 0 || tiles[x+y*terrainMap.width].terrainType == 2) disable = true;

    returnHome = true;
    foundPath = true;
    homeX = hx_;
    homeY = hy_;
    jobX = jx_;
    jobY = jy_;

    food = 0;
    minerals = 0;
  }


  void move()
  {
    super.move();
    
    // If this Person's jobType is a farmer:
    if (jobType == 0)
    {

      if (x == jobX && y == jobY)  // If this person is at their job location...
      {
        // It's possible that this worker's job location may have been settled while they were gone. If this
        // Is so, turn them back into a regular person and set their home to the new settlement.
        if ( tiles[loc].terrainType == 4 ) {
          dudes.add( new Person( x, y, x, y) );
          disable = true;
        } else {
          food += tiles[loc].fertility;  // ...gather resources...
          returnHome = true;  // ...and return home.
        }
      }

      if (x == homeX && y == homeY)  // If this person is at their home...
      {
        tiles[loc].storedFood += food;  // ...add their carried resources to their home's stored food.
        food = 0;   
        if (tiles[loc].storedFood >= tiles[loc].spawnThreshold) tiles[loc].spawn();  // Spawn a new Person if necessary
        returnHome = false;  // ...and return to their job location.
//        if (tiles[loc].settlementLevel == 0)
//        {
//          jobType = 0;
//        }
      }

      if      (returnHome == true)  goTo(homeX, homeY);
      else if (returnHome == false) goTo( jobX, jobY);
    }  // jobType = 1 end

    // If this Person's jobType is a miner:
    else if (jobType == 1)
    {

      if (x == jobX && y == jobY)  // If this person is at their job location...
      {
        // It's possible that this worker's job location may have been settled while they were gone. If this
        // Is so, turn them back into a regular person and set their home to the new settlement.
        if ( tiles[loc].terrainType == 4 ) {
          dudes.add( new Person( x, y, x, y) );
          disable = true;
        } else {
          minerals += tiles[loc].minWealth;  // ...gather resources...
          returnHome = true;  // ...and return home.
        }
      }

      if (x == homeX && y == homeY)  // If this person is at their home...
      {
        tiles[loc].storedMinerals += minerals;  // ...add their carried resources to their home's stored minerals.
        if (tiles[loc].storedMinerals >= tiles[loc].pioneerThreshold) tiles[loc].pioneer();  // Spawn a new Pioneer if necessary
        minerals = 0;   
        returnHome = false;  // ...and return to their job location.
//        if (tiles[loc].settlementLevel == 0)
//        {
//          jobType = 0;
//        }
      }

      if      (returnHome == true)  goTo(homeX, homeY);
      else if (returnHome == false) goTo( jobX, jobY);
    }  // jobType = 2 end
  }  // move() end


  // Function to move towards a certain point
  void goTo(int destX, int destY)
  {
    // Create movement probability matrix and fill each entry with 0
    int[][] moveProb = new int[3][3];
    for (int i = 0; i < 3; i++)
    {
      for (int j = 0; j < 3; j++)
      {
        int tileLoc = (x + (i-1)) + (y + (j-1))*terrainMap.width;  // 1D location of the tile being looked at
        
        if (tiles[tileLoc].terrainType != 0 && tiles[tileLoc].terrainType != 2)
        {  // If the tile has been moved on before
          moveProb[i][j] = 1;  // ...It can be moved to.
        } else moveProb[i][j] = 0;
      }
    }
    
    // Find a possible target location
    int possibleTargetX;
    int possibleTargetY;
    
    // If the last path was successful
    if (foundPath == true) {
      if (destX > x) possibleTargetX = 1;
      else if (destX < x) possibleTargetX = -1;
      else possibleTargetX = 0;
      
      if (destY > y) possibleTargetY = 1;
      else if (destY < y) possibleTargetY = -1;
      else possibleTargetY = 0;
    } else {
      // Get a new target location
      possibleTargetX = int( random(-2, 2) );
      possibleTargetY = int( random(-2, 2) );
    }
    
    // Test path
    if ( moveProb[possibleTargetX+1][possibleTargetY+1] == 0 ) foundPath = false;
    else foundPath = true;
    
    if (foundPath == true) {
      x += possibleTargetX;
      y += possibleTargetY;
    }
  }  // end of function

}  // end class
