PImage generateTerrain(PImage source, float waterThreshold, float landThreshold)
{
  PImage destination;  // The image which this function will create
  destination = createImage(source.width, source.height, RGB);
  
  int terrainType = 0;

  if (invert)
  {
    source.filter(INVERT);
  }

  source.loadPixels();
  destination.loadPixels();

  // Going through the image pixel-by-pixel
  for (int x = 0; x < source.width; x++)
  {
    for (int y = 0; y < source.height; y++)
    {     
      
      // Find 1D pixel location
      int loc = x + y*source.width;

      // Find rgb values of current pixel
      float r = red  (source.pixels[loc]);
      float g = green(source.pixels[loc]);
      float b = blue (source.pixels[loc]);

      // Find the average of the current pixel's rgb
      float currentAvg = (r + g + b) / 3;

      //currentAvg = map(currentAvg, 0, 255, 0, avgBright);

      // Compare the brightness of this pixel to the brightness of the total image
      // and apply a color accordingly
      int water_distance = 15;
      if (currentAvg < waterThreshold || (x <= water_distance || x >= width-water_distance) || (y <= water_distance || y >= height-water_distance) )
      { // If brightness is below 1/3 of the average (or it is within a certain number of tiles of the edge of the window), give it water colors
        r = map(currentAvg, 0, waterThreshold, 65, 76);
        g = map(currentAvg, 0, waterThreshold, 70, 95);
        b = map(currentAvg, 0, waterThreshold, 210, 213);
        destination.pixels[loc] = color(r, g, b);

        // Also set its terrain type to 'water'
        terrainType = 0;  // 0 = water
      }
      else if (currentAvg < landThreshold)
      { // If brightness is below 2/3 of the average, set its color to green
        r = map(currentAvg, waterThreshold, landThreshold, 21, 130);
        g = map(currentAvg, waterThreshold, landThreshold, 122, 154);
        b = map(currentAvg, waterThreshold, landThreshold, 42, 8);
        destination.pixels[loc] = color(r, g, b);

        // Also set its terrain type to 'land'
        terrainType = 1;  // 1 = land
      }
      else
      { // If brightness is above 2/3 of the average, sit its color to red
        r = map(currentAvg, landThreshold, 255, 152, 182);
        g = map(currentAvg, landThreshold, 255, 113, 181);
        b = map(currentAvg, landThreshold, 255,  51, 179);
        destination.pixels[loc] = color(r, g, b);

        // Also set its terrain type to 'mountains'
        terrainType = 2;  // 2 = mountains
      }  // end if chain
      
      // Create a new Tile with corresponding location and terrain type
      tiles[loc] = new Tile(x, y, terrainType, color(r,g,b));
      
    }  // end for y
  }  // end for x

  destination.updatePixels();
  return destination;
} // end function
